package com.dzeio.charts.series

import android.graphics.Canvas
import android.graphics.Paint
import android.graphics.RectF
import android.util.Log
import kotlin.math.max

class BarSerie : SerieAbstract() {

    companion object {
        const val TAG = "DzeioCharts/BarSerie"
    }

    var spacing: Float = 8f

    var targetPercentList = arrayListOf<Float>()
    var percentList = arrayListOf<Float>()

    private var fgPaint: Paint = Paint().also {
        it.isAntiAlias = true
    }

    var previousRefresh = 0

    override fun onUpdate(): Boolean {
        var needNewFrame = false
        for (i in targetPercentList.indices) {
            val value = view.animation.updateValue(
                1f,
                targetPercentList[i],
                percentList[i],
                0f,
                0.01f
            )

            if (value != percentList[i]) {
                needNewFrame = true
                percentList[i] = value
            }
        }
        return needNewFrame
    }

    override fun prepareData() {
        val max: Float = if (view.yAxis.max != null) view.yAxis.max!! else {
            getMax()
        }

        targetPercentList = arrayListOf()

//        Log.d(TAG, "offset: ${view.getXOffset()}, displayed: ${view.getDisplayedEntries()}")
        for (item in datas.subList(
            view.getXOffset(),
            view.getXOffset() + view.getDisplayedEntries()
        )) {
//            // // Process bottom texts
//            val text = view.xAxis.onValueFormat(item.x)
//            bottomTexts.add(text)
//
//            // get Text boundaries
//            view.xAxis.labels.build().getTextBounds(text, 0, text.length, r)
//
//            // get height of text
//            if (bottomTextHeight < r.height()) {
//                bottomTextHeight = r.height()
//            }
//
//            // get text descent
//            val descent = abs(r.bottom)
//            if (bottomTextDescent < descent) {
//                bottomTextDescent = descent
//            }

            // // process values
//            Log.d(TAG, item.y.toString())

            // add to animations the values
            targetPercentList.add(1 - item.y / max)
        }

        // post list
        val offset = view.getXOffset()
        val movement = offset - previousRefresh
        Log.d(TAG, "$offset - $previousRefresh = $movement")
        if (movement != 0) {
            previousRefresh = offset
        }
//        if (movement != 0) {
//            Log.d(TAG, movement.toString())
//        }
        if (movement >= 1) {
            percentList = percentList.subList(1, percentList.size).toCollection(ArrayList())
            percentList.add(1f)
        } else if (movement <= -1) {
            percentList = percentList.subList(0, percentList.size - 1).toCollection(ArrayList())
            percentList.add(0, 1f)
        }

        if (percentList.isEmpty() || percentList.size < targetPercentList.size) {
            val temp = targetPercentList.size - percentList.size
            for (i in 0 until temp) {
                percentList.add(1f)
            }
        } else if (percentList.size > targetPercentList.size) {
            val temp = percentList.size - targetPercentList.size
            for (i in 0 until temp) {
                percentList.removeAt(percentList.size - 1)
            }
        }

        fgPaint.color = view.yAxis.color
    }

    override fun displayData(canvas: Canvas, rect: RectF) {
        val barWidth = rect.width() / view.getDisplayedEntries() - spacing

        if (percentList.isNotEmpty()) {
            // draw each rectangles
            for (i in 1..percentList.size) {
//                Log.d(TAG, percentList[i - 1].toString())
                val left = rect.left + spacing * i + barWidth * (i - 1).toFloat()
//                Log.d(TAG, "$spacing, $i, $barWidth = $left")
                val right = rect.left + (spacing + barWidth) * i.toFloat()
                val bottom = rect.top + rect.height()
                val top = (bottom - rect.top) * percentList[i - 1]

                // create rounded rect
                canvas.drawRoundRect(left, top, right, bottom, 8f, 8f, fgPaint)
                // remove the bottom corners DUH
                canvas.drawRect(left, max(top, bottom - 8f), right, bottom, fgPaint)
                if (view.debug) {
                    canvas.drawText(
                        (getMax() - getMax() * targetPercentList[i - 1]).toString(),
                        left + (right - left) / 2,
                        top + (bottom - top) / 2,
                        view.xAxis.labels.build()
                    )
                }
            }
        }
    }

    private fun getMax(): Float {
        var calculatedMax = 0f
        for (entry in datas.subList(
            view.getXOffset(),
            view.getDisplayedEntries() + view.getXOffset()
        )) {
            if (entry.y > calculatedMax) calculatedMax = entry.y
        }
        return if (calculatedMax < 0) 0f else calculatedMax
    }
}
